/*
 * maintains QSO Histories and autoselect
 * ES1JA Nov 2017
 */

#include "qsohistory.h"
void QsoHistory::init()
{
    _data.clear();
    _blackdata.clear();
    _working = true;
    _CQ.call = "";    
    _CQ.status = NONE;
    _CQ.srx_c = NONE;
    _CQ.srx_p = NONE;
    _CQ.stx_c = NONE;
    _CQ.stx_p = NONE;
    _CQ.count = 0;
    _CQ.b_time = 0;
    _CQ.time = 0;
    _CQ.rx = 0;
    _CQ.distance = 0;
}

QsoHistory::latlng QsoHistory::fromQth(QString const& qth) {
    latlng mylatlng;
    mylatlng.lat=0;
    mylatlng.lng=0;
    QString myqth;
    auto qthLen = qth.size();
    if (qthLen < 4)  myqth = qth.left(2).toUpper() + "55LL55LL";
    else if (qthLen < 6)  myqth = qth.left(4).toUpper() + "LL55LL";
    else if (qthLen < 8)  myqth = qth.left(6).toUpper() + "55LL";
    else if (qthLen < 10) myqth+= qth.left(8).toUpper() + "LL";
    else myqth = qth.toUpper();
    if (_gridRe.match(myqth).hasMatch()) {
      int l [10];
      for(int i=0; i < myqth.length(); i++) l[i] = myqth.at(i).toLatin1() - 65; 
      l[2] += 17; 
      l[3] += 17;
      l[6] += 17; 
      l[7] += 17;
      mylatlng.lng = (l[0]*rad_0 + l[2]*rad_2 + l[4]*rad_4 + l[6]*rad_6 + (l[8]+0.5)*rad_8 - M_PI);
      mylatlng.lat = (l[1]*rad_1 + l[3]*rad_3 + l[5]*rad_5 + l[7]*rad_7 + (l[9]+0.5)*rad_9 - M_PI_2);
    }    
    return mylatlng;
}

int QsoHistory::Distance(latlng latlng1,latlng latlng2) {
    if ((latlng1.lat == 0 && latlng1.lng  == 0) || (latlng2.lat == 0 && latlng2.lng  == 0)) return 0;
    else {
        double dlon = latlng2.lng - latlng1.lng;
        double dlat = latlng2.lat - latlng1.lat;
        double a = qSin(dlat / 2.0) * qSin(dlat / 2.0) + qCos(latlng1.lat) *
                  qCos(latlng2.lat) * qSin(dlon / 2.0) * qSin(dlon / 2.0);
        double c = 2.0 * qAtan2(qSqrt(a), qSqrt(1.0 - a));
        return 6378.137 * c;
    }
}


int QsoHistory::remove(QString const& callsign)
{
  int ret=0,ret2=0;
  if (_working) {
    ret=_data.remove(Radio::base_callsign (callsign));
    ret2=_blackdata.remove(Radio::base_callsign (callsign));
  }
/*  printf("%s QSODele:%s (%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
    Radio::base_callsign (callsign).toStdString().c_str(),ret,ret2);*/
  return ret;
}

int QsoHistory::blacklist(QString const& callsign)
{
  int ret=0;
  if (_working) {
    ret = _blackdata.value(Radio::base_callsign (callsign),ret);
    ret += 1;
    _blackdata.insert(Radio::base_callsign (callsign),ret);
  }
/*  printf("%s QSOBlack:%s (%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
    Radio::base_callsign (callsign).toStdString().c_str(),ret,_blackdata.size());*/
  return ret;
}

int QsoHistory::reset_count(QString const& callsign)
{
  int ret=0;
  if (_working) {
        QSO t;
        t = _data.value(Radio::base_callsign (callsign),t);
        if (t.call == callsign) {
            t.count = 0;
            _data.insert(Radio::base_callsign (callsign),t);
            ret = 1;
        }
  }
/*  printf("%s QSOReset_count:%s (%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
    Radio::base_callsign (callsign).toStdString().c_str(),ret);*/
  return ret;
}

int QsoHistory::autoseq(QString &callsign, QString &grid, Status &status, QString &rep, int &rx, int &tx, unsigned &time, int &count)
{
    if (_working)
    {
      int ret = 0;
      if (callsign.length() > 2)
        {
          int hound = count;
          QSO t;
          t.status = NONE;
          t = _data.value(Radio::base_callsign (callsign),t);
          if (t.status > NONE) {
            status = t.status;
            if (hound == -1) {
              if(t.stx_c == SCALL && status == RREPORT) count = 1;
              else count = t.count + 1;
            } else count = t.count;
            if (t.grid.length() >3) grid = t.grid;
            if (!t.s_rep.isEmpty ()) rep = t.s_rep;
            if (t.rx >0) rx = t.rx;
            if (t.tx >0) tx = t.tx;
            if (t.b_time > 0) time = t.b_time;
            if ((status >= S73 || status == SRR73) && tx == _CQ.tx) t.direction = 1;
            ret = t.direction;
            if (hound == -1 && status == RRR73) {
              t.status = FIN;
              _data.insert(Radio::base_callsign (callsign),t);
            }
          }
/*          printf("%s QSOAuto:%s (%s,%s,%d,%s,%s,%d,%u,%u,%d,%d,%d,%d,%d,%d)(%s,%d,%s,%d,%d,%u,%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
            Radio::base_callsign (callsign).toStdString().c_str(),t.call.toStdString().c_str(),t.grid.toStdString().c_str(),t.distance,
            t.r_rep.toStdString().c_str(),t.s_rep.toStdString().c_str(),t.status,t.b_time,t.time,t.rx,t.tx,t.count,t.priority,t.direction,as_active,grid.toStdString().c_str(),status,rep.toStdString().c_str(),
            rx,tx,time,count,_data.size());*/
//          as_active = false;
          return ret;
        }
      else
        { 
          algo=time;
          dist = 0;
          if (algo & 128) a_init=-1;
          else a_init=0;
          if (algo & 64) b_init=-1;
          else b_init=0;
          if (as_active && _data.size() > 0) { //my CQ answers && _CQ.count > 0 
            QSO tt,t;
            int priority = a_init;
            rep = "-60";
            dist = 0;
            Rrep = "-60";
            foreach(QString key,_data.keys()) {
              ret=_blackdata.value(key,0);
              tt=_data[key];
              if (ret == 0 && tt.time == max_r_time && (tt.status == RCALL || tt.status == RREPORT) && tt.continent != "  ") {
                if (tt.priority > priority || (priority > a_init && tt.priority == priority 
                      && ((!(algo & 32) && ((!(algo & 16) && !tt.s_rep.isEmpty () && tt.s_rep.toInt() > rep.toInt())
                                            || (algo & 16 && ((tt.status == RCALL && !tt.s_rep.isEmpty () && tt.s_rep.toInt() > rep.toInt() && Rrep == "-60")
                                                              ||(tt.status == RREPORT && !tt.s_rep.isEmpty () && tt.s_rep.toInt() > Rrep.toInt()))))) 
                          || (algo & 32 && tt.distance > dist)))) {
                  if(_CQ.tyyp.isEmpty () || (_strictdirCQ && tt.priority > 4) || _CQ.tyyp == tt.continent || _CQ.tyyp == tt.mpx || _CQ.tyyp == tt.call.left(2) || (_CQ.tyyp == "DX" && tt.continent != mycontinent_)) {
                    t = tt;
                    priority = tt.priority;
                    status = tt.status;
                    callsign = tt.call;
                    count = tt.count;
                    dist = tt.distance;
                    if (tt.grid.length() >3) grid = tt.grid;
                    if (!tt.s_rep.isEmpty ()) {
                      if (tt.status == RREPORT && (algo & 16)) Rrep = tt.s_rep;
                      rep = tt.s_rep;
                    }
                    if (tt.rx >0) rx = tt.rx;
                    if (tt.tx >0) tx = tt.tx;
                    if (tt.status == RREPORT && tt.direction == 1) tt.b_time = max_r_time;
                    time = tt.b_time;
                    as_active = false;
                  }
                }
              }
            }
            if (!as_active) {
/*              printf("%s QSOSel1:%s (%s,%s,%d,%s,%s,%d,%u,%u,%d,%d,%d,%d,%d,%d)(%s,%d,%s,%d,%d,%u,%u,%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
                  Radio::base_callsign (callsign).toStdString().c_str(),t.call.toStdString().c_str(),t.grid.toStdString().c_str(),t.distance,
                  t.r_rep.toStdString().c_str(),t.s_rep.toStdString().c_str(),t.status,t.b_time,t.time,t.rx,t.tx,t.count,t.priority,t.direction,as_active,grid.toStdString().c_str(),status,rep.toStdString().c_str(),
                  rx,tx,time,max_r_time,count,_data.size());*/
              return t.direction;    
            }
          }
          if (algo&1 && as_active && _data.size() > 0){ // their CQ answers

            QSO tt,t;
            int priority = b_init;
            rep = "-60";
            dist = 0;
            foreach(QString key,_data.keys()) {
              ret=_blackdata.value(key,0);
              tt=_data[key];
              if (ret == 0 && tt.time == max_r_time && tt.status == RCQ && tt.continent != "  ") {
                if (tt.priority > priority || (priority > b_init && tt.priority == priority && ((!(algo & 32) && !tt.s_rep.isEmpty () && tt.s_rep.toInt() > rep.toInt()) || (algo & 32 && tt.distance > dist)))) {
                  if((tt.tyyp.isEmpty () || tt.tyyp.size () == 3 || (_strictdirCQ && tt.priority > 4) || tt.tyyp == mycontinent_ || tt.tyyp == _CQ.call.left(2) || tt.tyyp == myprefix_ || (tt.tyyp == "DX" && tt.continent != mycontinent_))
                  && (_CQ.tyyp.isEmpty () || _CQ.tyyp == tt.continent || (_strictdirCQ && tt.priority > 4) || _CQ.tyyp == tt.call.left(2) || (_CQ.tyyp == "DX" && tt.continent != mycontinent_))) {
                    t = tt;
                    priority = tt.priority;
                    status = tt.status;
                    callsign = tt.call;
                    count = tt.count;
                    dist = tt.distance;
                    if (tt.grid.length() >3) grid = tt.grid;
                    if (!tt.s_rep.isEmpty ()) rep = tt.s_rep;
                    if (tt.rx >0) rx = tt.rx;
                    if (tt.tx >0) tx = tt.tx;
                    time = tt.b_time;
                  }
                }
              }
            }
            as_active = false;
/*            printf("%s QSOSel2:%s (%s,%s,%d,%s,%s,%d,%u,%u,%d,%d,%d,%d,%d,%d)(%s,%d,%s,%d,%d,%u,%u,%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
                Radio::base_callsign (callsign).toStdString().c_str(),t.call.toStdString().c_str(),t.grid.toStdString().c_str(),t.distance,
                t.r_rep.toStdString().c_str(),t.s_rep.toStdString().c_str(),t.status,t.b_time,t.time,t.rx,t.tx,t.count,t.priority,t.direction,as_active,grid.toStdString().c_str(),status,rep.toStdString().c_str(),
                rx,tx,time,max_r_time,count,_data.size());*/
            return t.direction;    
            
          }
        }
    }
/*printf("%s QSOFail:%s (%d)(%s,%d,%s,%d,%d,%u,%u,%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
            callsign.toStdString().c_str(),as_active,grid.toStdString().c_str(),status,rep.toStdString().c_str(),
            rx,tx,time,max_r_time,count,_data.size());*/
as_active = false;
return 0;
}

void QsoHistory::time(unsigned time)
{
    if (_working) {
//        printf("%s QSOHist:time (%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),time);
        max_r_time = time;
    }
}

QsoHistory::Status QsoHistory::status(QString const& callsign, QString &grid)
{
    if (_working)
    {
        QSO t;
        t.call = callsign;
        t = _data.value(Radio::base_callsign (callsign),t);
        grid = t.grid;
//        printf("%s QSOHist:status (%s,%d,%s)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),callsign.toStdString().c_str(),t.status,grid.toStdString().c_str());
        return t.status;
    } else return NONE;
}

void QsoHistory::owndata(QString const& mycontinent, QString const& myprefix, QString const& mygrid, bool strictdirCQ)
{
    if (_working)
    {
        _strictdirCQ = !strictdirCQ;
        mycontinent_=mycontinent;
        myprefix_ = myprefix;
        _mylatlng = fromQth(mygrid);
//        printf("%s QSOHist:owndata (%s,%s,%s,%d) %f,%f\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),mycontinent_.toStdString().c_str(),
//          myprefix_.toStdString().c_str(),mygrid.toStdString().c_str(),_strictdirCQ,_mylatlng.lat,_mylatlng.lng);
    }
}

void QsoHistory::rx(QString const& callsign,int freq)
{
    if (_working)
    {
        QSO t;
        t.call = callsign;
        t.status = NONE;
        t.rx = 0;
        t = _data.value(Radio::base_callsign (callsign),t);
        if (t.status > NONE && t.rx != freq) {
            t.rx = freq;
            _data.insert(Radio::base_callsign (callsign),t);
        }
//        printf("%s QSOHist:rx (%s,%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),callsign.toStdString().c_str(),t.status,t.rx);
    }
}

void QsoHistory::message(QString const& callsign, Status status, int priority, QString const& param, QString const& tyyp, QString const& continent, QString const& mpx, unsigned time, QString const& rep, int freq)
{
    if (_working)
    {
      if (status == SCQ)
        {
          _CQ.call = callsign;
          _CQ.status = status;
          _CQ.stx_p = _CQ.stx_c;
          _CQ.stx_c = status;
          _CQ.grid = param;
          _CQ.tyyp = tyyp;
          _CQ.continent = mycontinent_;
          _CQ.mpx = myprefix_;
          _CQ.time = _CQ.b_time = time;
          _CQ.tx = freq;
          _CQ.priority = priority;
          _CQ.count += 1;
          _CQ.direction = 1;
/*          printf("%s QSOHist:CQ (%s,%s,%s,%s,%s,%s,%s,%d,%u,%u,%d,%d,%d,%d,%d)(%d,%d,%s,%s,%s,%s,%d,%s,%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
            _CQ.call.toStdString().c_str(),_CQ.grid.toStdString().c_str(),_CQ.tyyp.toStdString().c_str(),_CQ.continent.toStdString().c_str(),_CQ.mpx.toStdString().c_str(),
            _CQ.r_rep.toStdString().c_str(),_CQ.s_rep.toStdString().c_str(),_CQ.status,_CQ.b_time,_CQ.time,_CQ.rx,_CQ.tx,_CQ.count,_CQ.priority,_CQ.direction,status,priority,param.toStdString().c_str(),
            tyyp.toStdString().c_str(),continent.toStdString().c_str(),mpx.toStdString().c_str(),time,rep.toStdString().c_str(),freq,_data.size());*/
        }
      else
        {
          Status old_status = NONE;
          QSO t;
          t.status = NONE;
          t.srx_c = NONE;
          t.srx_p = NONE;
          t.stx_c = NONE;
          t.stx_p = NONE;
          t.count = 0;
          t.priority = 0;
          t.tx = 0;
          t.rx = 0;
          t.b_time = 0;
          t.call = callsign;
          t.direction = 0;
          t.time = 0;
          t.tyyp = "";
          t.continent = "";
          t.mpx = "";
          t.grid = "";
          t.distance = 0;
          t = _data.value(Radio::base_callsign (callsign),t);
          t.time = time;

          if (status == SCALL || status == SREPORT || status == SRR || status == SRR73 || status == S73) {
              if (status == t.stx_c) t.count += 1;
              else {
                t.count = 1;
//                printf ("reset count to 1 %d,%d,%d\n",status,t.status,t.direction); 
              }
              _CQ.count = 0;
          } else if (status != t.srx_c) {
//            printf ("reset count %d to 0 %d,%d,%d\n",t.count,t.srx_c,status,t.direction); 
            t.count = 0;
          }

          if (t.call.length() < callsign.length() && callsign.contains(t.call))
            {
              t.call=callsign;
            }
          switch (status)
          {
            case RCQ:
              {
                t.srx_p = t.srx_c;
                t.srx_c = status;
//                if (t.status <= SREPORT)
                if (t.status <= SCQ || t.status == SCALL || (t.status == RCALL && t.time != time)) // an attempt to support CQ and any other message reception from MSHV multislot operation mode
                  {
                    old_status = t.status;
                    t.status = status;
                    t.priority = priority;
                    t.grid = param;
                    t.tyyp = tyyp;
                    t.continent = continent;
                    t.mpx = mpx;
                    t.b_time = time;
                    t.s_rep = rep;
                    t.rx = freq;
                    t.direction = 0;
                    t.distance=Distance(_mylatlng,fromQth(param));
                  }  
//                max_r_time = time;
                as_active = true;
                break;
              }
            case RCALL:
              {
                t.srx_p = t.srx_c;
                t.srx_c = status;
                t.direction = 1;
                old_status = t.status;
                t.status = status;
                t.priority = priority;
                t.grid = param;
                t.continent = continent;
                t.mpx = mpx; 
                t.b_time = time;
                t.s_rep = rep;
                t.rx = freq;
                t.distance=Distance(_mylatlng,fromQth(param));
//                max_r_time = time;
                as_active = true;
                break;
              }
            case SCALL:
              {
                t.stx_p = t.stx_c;
                t.stx_c = status;
//                if (t.status <= SCALL ) {
                  old_status = t.status;
                  t.status = status;
                  t.b_time = time;
                  t.tx = freq;
                  t.direction = 0;
//                }
                break;
			  }
            case RREPORT:
              {
                t.srx_p = t.srx_c;
                t.srx_c = status;
                if  (t.status <= status || t.direction == 0) t.s_rep = rep;
                old_status = t.status;
                t.status = status;
                if (priority > t.priority) t.priority = priority;
                if (t.continent.isEmpty ()) t.continent = continent;
                if (t.mpx.isEmpty ()) t.mpx = mpx;
                t.r_rep = param;
                if (t.b_time == 0) t.b_time = time;
                t.rx = freq;
//                max_r_time = time;
                as_active = true;
                break;
              }
            case SREPORT:
              {
                t.stx_p = t.stx_c;
                t.stx_c = status;
                if (t.status <= SREPORT ) {
                  if (t.status <= RCQ) {
                    t.direction = 1;
                    t.s_rep = param;
                  } 
                  old_status = t.status;
                  t.status = status;
                  t.tx = freq;
                  if (t.b_time == 0) t.b_time = time;
                }
                break;
              }
            case RRR:
              {
                t.srx_p = t.srx_c;
                t.srx_c = status;
                if (!t.r_rep.isEmpty () && !t.s_rep.isEmpty () && t.status < FIN)
                  {
                    old_status = t.status;
                    t.status = status;
                    t.rx = freq;
                  }
//                max_r_time = time;
                break;
              }
            case SRR:
              {
                t.stx_p = t.stx_c;
                t.stx_c = status;
                if (t.status <= SRR ) {
                  old_status = t.status;
                  t.status = status;
                  t.tx = freq;
                }
                break;
              }
            case RRR73:
              {
                t.srx_p = t.srx_c;
                t.srx_c = status;
                if(!t.r_rep.isEmpty () && !t.s_rep.isEmpty () && t.status >= SREPORT)
                  {
                    if (t.status == SRR73 || (t.srx_p != RRR73 && t.status == FIN))
                      {
                        old_status = t.status;
                        t.status = FIN;
                        t.priority = 0;
                      }
                    else 
                      {
                        old_status = t.status;
                        t.status = status;
                      }
                    t.rx = freq;
                  }
//                max_r_time = time;
                break;
              }
            case SRR73:
              {
                t.stx_p = t.stx_c;
                t.stx_c = status;
                if (t.status == RRR73 || t.status == R73 || t.status == FIN)
                  {
                    old_status = t.status;
                    t.status = FIN;
                    t.priority = 0;
                  }
                else
                  {
                    old_status = t.status;
                    t.status = status;
                  }
                t.tx = freq;
                break;
              }
            case R73:
              {
                t.srx_p = t.srx_c;
                t.srx_c = status;
                if(!t.r_rep.isEmpty () && !t.s_rep.isEmpty () && t.status >= SREPORT)
                  {
                    if (t.status == SRR73 || t.status == S73 || t.status == FIN)
                      {
                        old_status = t.status;
                        t.status = FIN;
                        t.priority = 0;
                      }
                    else
                      {
                        old_status = t.status;
                        t.status = status;
                      }
                    t.rx = freq;
                  }
//                max_r_time = time;
                break;
              }
            case S73:
              {
                t.stx_p = t.stx_c;
                t.stx_c = status;
                if (t.status == RRR73 || t.status == R73 || t.status == FIN)
                  {
                    old_status = t.status;
                    t.status = FIN;
                    t.priority = 0;
                  }
                else
                  {
                    old_status = t.status;
                    t.status = status;
                  }
                t.tx = freq;
                break;
              }
            default:
              {
                old_status = t.status;
                t.status = status;
                break;
              }
          }
          _data.insert(Radio::base_callsign (callsign),t);
/*          printf("%s QSOHist:%s (%s,%s,%d,%s,%s,%s,%s,%s,%d->%d,%u,%u,%d,%d,%d,%d,%d)(%d,%d,%s,%s,%s,%s,%d,%s,%d,%d)\n",QDateTime::currentDateTimeUtc().toString("hh:mm:ss.zzz").toStdString().c_str(),
            Radio::base_callsign (callsign).toStdString().c_str(),t.call.toStdString().c_str(),t.grid.toStdString().c_str(),t.distance,t.tyyp.toStdString().c_str(),t.continent.toStdString().c_str(),
            t.mpx.toStdString().c_str(),t.r_rep.toStdString().c_str(),t.s_rep.toStdString().c_str(),old_status,t.status,t.b_time,t.time,t.rx,t.tx,t.count,t.priority,t.direction,status,priority,param.toStdString().c_str(),
            tyyp.toStdString().c_str(),continent.toStdString().c_str(),mpx.toStdString().c_str(),time,rep.toStdString().c_str(),freq,_data.size());*/
        }
    }
      
}     
      

